%module sentencepiece

%{
#include <sentencepiece_processor.h>

namespace {
PyObject* kStringInput = reinterpret_cast<PyObject* >(0x1);
PyObject* kUnicodeInput = reinterpret_cast<PyObject* >(0x2);

class PyInputString {
 public:
  explicit PyInputString(PyObject* obj) {
#if PY_VERSION_HEX >= 0x03000000
    if (PyUnicode_Check(obj)) {
      str_ = PyUnicode_AsUTF8AndSize(obj, &size_);
      input_type_ = kUnicodeInput;
    } else if (PyBytes_Check(obj)) {
      PyBytes_AsStringAndSize(obj, &str_, &size_);
      input_type_ = kStringInput;
    }
#else
    if (PyUnicode_Check(obj)) {
      utf8_obj_ = PyUnicode_AsUTF8String(obj);
      PyString_AsStringAndSize(utf8_obj_, &str_, &size_);
      input_type_ = kUnicodeInput;
    } else if (PyString_Check(obj)) {
      PyString_AsStringAndSize(obj, &str_, &size_);
      input_type_ = kStringInput;
    }
#endif
    else {
      str_ = nullptr;
    }
  }
  virtual ~PyInputString() {
    Py_XDECREF(utf8_obj_);
  }
  const char* str() const { return str_; }
  Py_ssize_t size() const { return size_; }
  bool IsAvalable() const { return str_ != nullptr; }
  PyObject *input_type() const { return input_type_; }

 private:
  PyObject* utf8_obj_ = nullptr;
  PyObject* input_type_ = nullptr;
  char* str_ = nullptr;
  Py_ssize_t size_ = 0;
};

PyObject* MakePyOutputString(const std::string& output, PyObject *resultobj) {
#if PY_VERSION_HEX >= 0x03000000
  return resultobj == kStringInput ?
      PyBytes_FromStringAndSize(output.data(), output.size()) :
      PyUnicode_FromStringAndSize(output.data(), output.size());
#else
   return resultobj == kUnicodeInput ?
       PyUnicode_FromStringAndSize(output.data(), output.size()) :
       PyString_FromStringAndSize(output.data(), output.size());
#endif
}
}  // namespace
%}

%ignore sentencepiece::SentencePieceProcessor::Encode(std::string const &, std::vector<std::string>*) const;
%ignore sentencepiece::SentencePieceProcessor::Encode(std::string const &, std::vector<int>*) const;
%ignore sentencepiece::SentencePieceProcessor::Encode(std::string const &, SentencePieceText *) const;
%ignore sentencepiece::SentencePieceProcessor::Decode(std::vector<std::string> const &,std::string *) const;
%ignore sentencepiece::SentencePieceProcessor::Decode(std::vector<int> const &, std::string *) const;
%ignore sentencepiece::SentencePieceProcessor::Decode(std::vector<std::string> const &, SentencePieceText *) const;
%ignore sentencepiece::SentencePieceProcessor::Decode(std::vector<int> const &, SentencePieceText *) const;
%ignore sentencepiece::SentencePieceProcessor::model_proto;
%ignore sentencepiece::SentencePieceProcessor::Load(std::istream *);
%ignore sentencepiece::SentencePieceProcessor::LoadOrDie(std::istream *);
%ignore sentencepiece::SentencePieceProcessor::model_proto();

%extend sentencepiece::SentencePieceProcessor {
  std::vector<std::string> Encode(const std::string& input) const {
    std::vector<std::string> output;
    $self->Encode(input, &output);
    return output;
  }

  std::vector<std::string> EncodeAsPieces(const std::string& input) const {
    std::vector<std::string> output;
    $self->Encode(input, &output);
    return output;
  }

  std::vector<int> EncodeAsIds(const std::string& input) const {
    std::vector<int> output;
    $self->Encode(input, &output);
    return output;
  }

  std::string Decode(const std::vector<std::string>& input) const {
    std::string output;
    $self->Decode(input, &output);
    return output;
  }

  std::string DecodePieces(const std::vector<std::string>& input) const {
    std::string output;
    $self->Decode(input, &output);
    return output;
  }

  std::string DecodeIds(const std::vector<int>& input) const {
    std::string output;
    $self->Decode(input, &output);
    return output;
  }

  int __len__() {
    return $self->GetPieceSize();
  }

  int __getitem__(const std::string& key) const {
    return $self->PieceToId(key);
  }
}

%typemap(out) std::vector<int> {
  $result = PyList_New($1.size());
  for (size_t i = 0; i < $1.size(); ++i) {
    PyList_SetItem($result, i, PyInt_FromLong(static_cast<long>($1[i])));
  }
}

%typemap(out) std::vector<std::string> {
  PyObject *input_type = resultobj;
  $result = PyList_New($1.size());
  for (size_t i = 0; i < $1.size(); ++i) {
    PyList_SetItem($result, i, MakePyOutputString($1[i], input_type));
  }
}

%typemap(out) std::string {
  PyObject *input_type = resultobj;
  $result = MakePyOutputString($1, input_type);
}

%typemap(in) const std::string & {
  const PyInputString ustring($input);
  if (!ustring.IsAvalable()) {
    PyErr_SetString(PyExc_TypeError, "not a string");
    return nullptr;
  }
  resultobj = ustring.input_type();
  $1 = new std::string(ustring.str(), ustring.size());
}

%typemap(in) const std::vector<std::string>& {
  std::vector<std::string> *out = nullptr;
  if (PyList_Check($input)) {
    const size_t size = PyList_Size($input);
    out = new std::vector<std::string>(size);
    for (size_t i = 0; i < size; ++i) {
      const PyInputString ustring(PyList_GetItem($input, i));
      if (ustring.IsAvalable()) {
        (*out)[i] = std::string(ustring.str(), ustring.size());
      } else {
        PyErr_SetString(PyExc_TypeError, "list must contain strings");
        return nullptr;
      }
      resultobj = ustring.input_type();
    }
  } else {
    PyErr_SetString(PyExc_TypeError, "not a list");
    return nullptr;
  }
  $1 = out;
}

%typemap(in) const std::vector<int>& {
  std::vector<int> *out = nullptr;
  if (PyList_Check($input)) {
    const size_t size = PyList_Size($input);
    out = new std::vector<int>(size);
    for (size_t i = 0; i < size; ++i) {
      PyObject *o = PyList_GetItem($input, i);
      if (PyInt_Check(o)) {
        (*out)[i] = static_cast<int>(PyInt_AsLong(o));
      } else {
        PyErr_SetString(PyExc_TypeError,"list must contain integers");
        return nullptr;
      }
    }
  } else {
    PyErr_SetString(PyExc_TypeError,"not a list");
    return nullptr;
  }
  $1 = out;
}

%typemap(freearg) const std::string& {
  delete $1;
}

%typemap(freearg) const std::vector<std::string>& {
  delete $1;
}

%typemap(freearg) const std::vector<int>& {
  delete $1;
}

%include <sentencepiece_processor.h>
